@@ -7,29 +7,29 @@ use Coro;
 use Coro::Channel;
 use Coro::Handle;
 use Coro::ProcessPool::Util qw(encode decode $EOL);
+use Guard qw(scope_guard);
 
 if ($^O eq 'MSWin32') {
     die 'MSWin32 is not supported';
 }
 
-my $IN     = unblock *STDIN;
-my $OUT    = unblock *STDOUT;
-my $outbox = Coro::Channel->new();
-my $outbox_mon;
+my $IN  = unblock *STDIN;
+my $OUT = unblock *STDOUT;
 
-sub start_outbox_mon {
-    $outbox_mon = async {
+sub start {
+    my $class   = shift;
+    my $outbox  = Coro::Channel->new();
+    my $monitor = async {
         while (1) {
             my $line = $outbox->get or last;
             $OUT->print($line . $EOL);
         }
     };
-}
 
-sub start {
-    my $class = shift;
-
-    $class->start_outbox_mon;
+    scope_guard {
+        $outbox->shutdown;
+        $monitor->join;
+    };
 
     while (1) {
         my $line  = $IN->readline($EOL) or last;
@@ -37,11 +37,6 @@ sub start {
         my $reply = $class->process_task($data);
         $outbox->put(encode($reply));
     }
-
-    $outbox->shutdown;
-    $outbox_mon->join;
-
-    exit 0;
 }
 
 sub process_task {
